package com.example.usb_printer;

import androidx.annotation.NonNull;

import android.annotation.SuppressLint;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.util.Log;

import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.Iterator;


import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel.EventSink;


/** UsbPrinterPlugin */
public class UsbPrinterPlugin implements FlutterPlugin, MethodCallHandler {

  private MethodChannel methodChannel;

  private static final String TAG = "UsbAdmin";
  private UsbManager mUsbManager;
  private UsbDevice mDevice;
  private UsbDeviceConnection mConnection;
  private UsbEndpoint mEndpointIntr;
  private static PendingIntent mPermissionIntent=null;
  private static final String ACTION_USB_PERMISSION="com.android.example.USB_PERMISSION";


  public static void registerWith(Registrar registrar) {
    final UsbPrinterPlugin instance = new UsbPrinterPlugin();
    instance.onAttachedToEngine(registrar.context(), registrar.messenger());
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
    methodChannel = new MethodChannel(binding.getFlutterEngine().getDartExecutor(), "usbPrinter");
    methodChannel.setMethodCallHandler(this);
    onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
  }

  private void onAttachedToEngine(Context context, BinaryMessenger messenger) {
    mUsbManager = (UsbManager)context.getSystemService(Context.USB_SERVICE);
    mPermissionIntent= PendingIntent.getBroadcast(context,0,new Intent(ACTION_USB_PERMISSION), 0);
    IntentFilter filter= new IntentFilter(ACTION_USB_PERMISSION);
    context.registerReceiver(mUsbReceiver, filter);
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    methodChannel.setMethodCallHandler(null);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
    case "listDevices":
      listDevices(result);
      break;
    case "openPort":
      int vid = (int) (call.argument("vid") == null ? 0 : call.argument("vid"));
      int pid = (int) (call.argument("pid") == null ? 0 : call.argument("pid"));
      int deviceId = (int) (call.argument("deviceId") == null ? 0 : call.argument("deviceId"));
      Openusb(vid,pid,deviceId);
      result.success(null);
      break;
    case "write":
      sendCommand((byte[]) call.argument("bytes"));
      result.success(null);
      break;
    default:
      result.notImplemented();
      break;
    }
  }



  @SuppressLint("NewApi")
    private void setDevice(UsbDevice device)
    {
        if(device!=null)
        {
            UsbInterface intf=null;
            UsbEndpoint ep=null;

            int InterfaceCount=device.getInterfaceCount();
            int j;

            mDevice = device;
            int i;

            intf = device.getInterface(0);
            Log.i(TAG,"interface:"+"class:"+intf.getInterfaceClass());
            int UsbEndpointCount=intf.getEndpointCount();
            for(i = 0;i < UsbEndpointCount;i++)
            {
                if (intf.getEndpoint(i).getType() == UsbConstants.USB_ENDPOINT_XFER_BULK && intf.getEndpoint(i).getDirection() == 0) {
                    ep=intf.getEndpoint(i);
                    break;
                }
            }


            mEndpointIntr = ep;
            if (device != null)
            {
                
                UsbDeviceConnection connection = mUsbManager.openDevice(device);

                if (connection != null && connection.claimInterface(intf, true)) {
                    Log.i(TAG,"open successed.");
                    mConnection = connection;

                } else {
                    Log.i(TAG,"open failed");
                    mConnection = null;
                }

                
            }
        }
        else
        {

        }
    }


  private UsbDevice getDevice(int vid, int pid, int deviceId) {
    Map<String, UsbDevice> devices = mUsbManager.getDeviceList();
    for (UsbDevice device : devices.values()) {
      if (deviceId == device.getDeviceId() || (device.getVendorId() == vid && device.getProductId() == pid)) {
        return device;
      }
    }
    return null;
  }

    @SuppressLint("NewApi")
    public void Openusb(int vid, int pid, int deviceId)
    {
        UsbDevice device = getDevice(vid, pid, deviceId);
        setDevice(device);
        if(mConnection==null)
        {
          mUsbManager.requestPermission(device, mPermissionIntent);
        }
    }


    @SuppressLint("NewApi")
    public void Closeusb()
    {
        if (mConnection != null)
        {
            mConnection.close();
            mConnection=null;
        }
    }

    public boolean GetUsbStatus()
    {
        if(mConnection==null)
            return false;
        return true;
    }


  private HashMap<String, Object> serializeDevice(UsbDevice device) {
    HashMap<String, Object> dev = new HashMap<>();
    dev.put("vid", device.getVendorId());
    dev.put("pid", device.getProductId());
    if (android.os.Build.VERSION.SDK_INT >= 21) {
      dev.put("manufacturerName", device.getManufacturerName());
      dev.put("productName", device.getProductName());
      dev.put("serialNumber", device.getSerialNumber());
    }
    dev.put("deviceId", device.getDeviceId());
    return dev;
  }

  private void listDevices(Result result) {
    Map<String, UsbDevice> devices = mUsbManager.getDeviceList();
    if (devices == null) {
      result.error("LIST_DEVICES_ERROR", "Could not get USB device list.", null);
      return;
    }
    List<HashMap<String, Object>> transferDevices = new ArrayList<>();

    for (UsbDevice device : devices.values()) {
      transferDevices.add(serializeDevice(device));
    }
    result.success(transferDevices);
  }

    @SuppressLint("NewApi")
    public boolean sendCommand(byte [] Content)
    {
        boolean Result;
        synchronized (this)
        {
            int len=-1;
            if (mConnection != null)
            {
                len=mConnection.bulkTransfer(mEndpointIntr, Content, Content.length, 10000);
            }

            if(len<0)
            {
                Result=false;
                Log.i(TAG,"    ʧ ܣ  "+len);
            }
            else
            {
                Result=true;
                Log.i(TAG,"    "+len+" ֽ     ");
            }
        }
        return Result;
    }
    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (ACTION_USB_PERMISSION.equals(action))
            {
                synchronized (this)
                {
                    UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false))
                    {
                        if(device != null)
                        {
                            setDevice(device);
                        }
                        else
                        {
                            Closeusb();
                            mDevice=device;
                        }
                    }
                    else
                    {
                        Log.d(TAG, "permission denied for device " + device);
                    }

                }

            }
        }
    };
}