import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:usb_printer/usb_printer.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {

  UsbPrinter usbPrinter = UsbPrinter();

  List<UsbDevice> _devices = [];
  final _scaffoldKey = GlobalKey<ScaffoldState>();


  @override
  void initState() {
    super.initState();
    getDeviceList();
  }


  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Test external printer'),
        ),
        body: Column(
          children: <Widget>[
            Padding(
              padding: const EdgeInsets.only(top: 15, bottom: 15),
              child: Text(
                  _devices.isNotEmpty
                      ? "Available USB Devices"
                      : "No USB devices available",
                  style: Theme.of(context).textTheme.title),
            ),
            ..._portList(),
            FlatButton(
              onPressed: (){
                String data = "nikhil\nnikhil\nnikhil";
                usbPrinter.write(bytes:Uint8List.fromList(data.codeUnits));
              }, 
              child: Text("click me")
            )
          ],
        ),
      ),
    );
  }

  List<Widget> _portList() {
    List<Widget> ports = [];
    _devices.forEach((device) {
      ports.add(
        ListTile(
          leading: Icon(Icons.usb),
          title: Text(device.productName),
          subtitle: Text(device.manufacturerName),
          trailing: RaisedButton(
            child: Text("Connect"),
            onPressed: () async {
              await _connect(device);       
            },
          ),
        ),
      );
    });
    if (ports.isEmpty) {
      ports.add(SizedBox.shrink());
    }
    return ports;
  }


  Future<UsbDevice> _connect(UsbDevice device) async {
    var result = await usbPrinter.open(
      vendorId: device.vid,
      productId: device.pid,
      deviceId: device.deviceId
    );
    return result;
  }

  void showSnackBar(String message) {
    final snackBar = SnackBar(
      content: Text(message),
    );
    _scaffoldKey.currentState.showSnackBar(snackBar);
  }


  Future getDeviceList() async{
    try {
      List<UsbDevice> devices = await usbPrinter.listDevices();
      setState(() {
        _devices = devices;
      });
    } on PlatformException catch (e) {
      showSnackBar(e.message);
    }
  }
}
