import 'dart:typed_data';

import 'package:flutter/services.dart';

class UsbPrinter{

  static const MethodChannel _channel = const MethodChannel('usbPrinter');

    /// Returns a list of UsbDevices currently plugged in.
  Future<List<UsbDevice>> listDevices() async {
    List<dynamic> devices = await _channel.invokeMethod("listDevices");
    return devices.map(UsbDevice.fromJSON).toList();
  }


  /// Opens connection to device 
  Future<UsbDevice> open({
    int deviceId,
    int vendorId,
    int productId,
  }) async {
    Map<String, dynamic> args = {
      "vid": vendorId,
      "pid": productId,
      "deviceId": deviceId
    };
    try {
      dynamic result = await _channel.invokeMethod("openPort",args);
      return result;
    } on PlatformException catch (e) {
      throw (e);
    }
  }


  /// Sends raw bytes to USB device.
  Future<bool> write({
    Uint8List bytes,
  })async {
    Map<String, dynamic> args = {
      "bytes": bytes,
    };
    try {
      return await _channel.invokeMethod("write", args);
    } on PlatformException catch (e) {
      throw (e);
    }
  }
}


/// This is used to determine which Usb Device to open.
class UsbDevice {
  /// Vendor Id
  final int vid;
  /// Product Id
  final int pid;
  final String productName;
  final String manufacturerName;
  /// The device id is unique to this Usb Device until it is unplugged.
  /// when replugged this ID will be different.
  final int deviceId;
  /// The Serial number from the USB device.
  final String serial;
  UsbDevice(this.vid, this.pid, this.productName, this.manufacturerName,
      this.deviceId, this.serial);
  static UsbDevice fromJSON(dynamic json) {
    return UsbDevice(json["vid"], json["pid"], json["productName"],
        json["manufacturerName"], json["deviceId"], json["serialNumber"]);
  }
  Map<String, dynamic> toJson() => {
        'vid': vid,
        'pid': pid,
        'productName': productName,
        'manufacturerName': manufacturerName,
        'deviceId': deviceId,
        'serialNumber': serial,
      };
  @override
  String toString() {
    return "UsbDevice: ${vid.toRadixString(16)}-${pid.toRadixString(16)} $productName, $manufacturerName $serial";
  }
}
